@@ -2,7 +2,6 @@ from hdx.location.adminone import AdminOne
 from hdx.utilities.dateparse import parse_date
 
 from hdx.scraper.base_scraper import BaseScraper
-from hdx.scraper.configurable.timeseries import TimeSeries
 from hdx.scraper.outputs.json import JsonFile
 from hdx.scraper.runner import Runner
 
@@ -17,11 +16,9 @@ class TestScrapersTimeSeries:
         name = "timeseries_casualties"
         jsonout = JsonFile(configuration["json"], [name])
         outputs = {"json": jsonout}
-        scrapers = TimeSeries.get_scrapers(
-            configuration["timeseries"], today, outputs
+        scraper_names = runner.add_timeseries_scrapers(
+            configuration["timeseries"], outputs
         )
-        runner.add_customs(scrapers)
-        scraper_names = runner.add_customs(scrapers, add_to_run=True)
         assert scraper_names == [name]
         runner.run()
         assert jsonout.json[f"{name}_data"] == [
